% this script provides numerical solutions to the game in finite
% population % user has to enter 
% - the number of players, 
% - the intensity of selection, and
% - payoff matrix [a b; c d]
% the script outputs fixation probability of mutants P, 
% time to absorption T and time to fixation F

%% inputs
N = 5;  % number of individuals
w = 1;  % intensity of selection, between 0 and 1
a = 3;  % reward to mutant when playing mutant,     should be >=0
b = 1;  % reward to mutant when playing resident,   should be >=0
c = 6;  % reward to resident when playing mutant,   should be >=0
d = 0;  % reward to resident when playing resident, should be >=0

%% preliminary calculations

% EM(i) will be payoff to mutants when there are i mutants
EM = ((1:(N-1))*(a-b) - a+b*N)/(N-1); 
% ER(i) will be payoff to residents when there are i mutants
ER = (1:(N-1))*(c-d)/(N-1) + d;

rM = 1-w+w*EM;  % fitness when selection is incorporated 
rR = 1-w+w*ER;  % fitness when selection is incorporated

% probability of mutants to increase from i to i+1
pInc = (1:(N-1)).*rM./((1:(N-1)).*rM +(N-(1:(N-1))).*rR).*(N-(1:(N-1)))/N;
% probability of mutants to decrease from i to i-1
pDec = (N-(1:(N-1))).*rR./((1:(N-1)).*rM +(N-(1:(N-1))).*rR).*(1:(N-1))/N;
% probability to stay on i
pStay= 1-pInc-pDec; 
    
% determine the matrix of linear equations
A = diag ([pStay-1 1]) + diag (pInc,1) + diag([pDec(2:(N-1)) 0],-1);
B=zeros(N,1); B(N,1)=1; % right hand side for fixation probability

%% calculations and outputs
p=A\B           % fixation probabilities solves A*p=B
t=A\(B-1)       % absorption times solve A*t=(B-1)
pf=A\((B-1).*p);% fixation times solve A*(p.*f)= (B-1).*p
f=pf./p         % solve for fixation times
% p(i) is fixation probability given there are i mutants now, 
% similarly for t(i) and f(i)
